@@ -1,5 +1,5 @@
 #!/usr/bin/python3
-# This file is part of Epoptes, http://epoptes.org
+# This file is part of Epoptes, https://epoptes.org
 # Copyright 2010-2018 the Epoptes team, see AUTHORS.
 # SPDX-License-Identifier: GPL-3.0-or-later
 """
@@ -42,8 +42,7 @@ class EpoptesGui(object):
         self.current_macs = subprocess.Popen(
             ['sh', '-c',
              r"""ip -oneline -family inet link show | """
-             r"""sed -n '/.*ether[[:space:]]*\([[:xdigit:]:]*\).*/"""
-             r"""{s//\1/;y/abcdef-/ABCDEF:/;p;}';"""
+             r"""sed -n 's/.*ether[[:space:]]*\([[:xdigit:]:]*\).*/\1/p';"""
              r"""echo $LTSP_CLIENT_MAC"""],
             stdout=subprocess.PIPE).communicate()[0].decode().split()
         self.current_thumbshots = dict()
@@ -114,8 +113,35 @@ class EpoptesGui(object):
             self.default_group_ref.get_path())
         self.get('adj_icon_size').set_value(self.ts_width)
         self.on_scl_icon_size_value_changed(None)
-        _saved_clients, groups = config.read_groups(
-            config.expand_filename('groups.json'))
+        # Support a global groups.json, writable only by the "administrator"
+        self.groups_file = '/etc/epoptes/groups.json'
+        if os.access(self.groups_file, os.R_OK):
+            # Don't use global groups for the "administrator"
+            self.global_groups = not os.access(self.groups_file, os.W_OK)
+        else:
+            self.groups_file = config.expand_filename('groups.json')
+            self.global_groups = False
+        try:
+            _saved_clients, groups = config.read_groups(self.groups_file)
+        except ValueError as exc:
+            self.warning_dialog(
+                _('Failed to read the groups file:') + '\n'
+                + self.groups_file + '\n'
+                + _('You may need to restore your groups from a backup!')
+                + '\n\n' + str(exc))
+            _saved_clients, groups = [], []
+        # In global groups mode, groups that start with X- are hidden
+        self.x_groups = {}
+        if self.global_groups:
+            for group in list(groups):
+                if group.name.upper().startswith('X-HIDDEN'):
+                    if 'X-HIDDEN' not in self.x_groups:
+                        self.x_groups['X-HIDDEN'] = []
+                    self.x_groups['X-HIDDEN'] = list(set(
+                        self.x_groups['X-HIDDEN']
+                        + [m.mac for m in group.members]))
+                if group.name.upper().startswith('X-'):
+                    groups.remove(group)
         if groups:
             self.mni_add_to_group.set_sensitive(True)
         for group in groups:
@@ -141,8 +167,8 @@ class EpoptesGui(object):
         sel_group = self.gstore.get_path(self.get_selected_group()[0])[0]
         self.gstore.remove(self.gstore.get_iter(
             self.default_group_ref.get_path()))
-        config.save_groups(config.expand_filename('groups.json'),
-                           self.gstore)
+        if not self.global_groups:
+            config.save_groups(self.groups_file, self.gstore)
         settings = config.settings
         if not settings.has_section('GUI'):
             settings.add_section('GUI')
@@ -279,9 +305,8 @@ class EpoptesGui(object):
             pwd = ''.join(random.sample(
                 string.ascii_letters + string.digits, 8))
             subprocess.call(['x11vnc', '-storepasswd', pwd, pwdfile])
-            file = open(pwdfile, 'rb')
-            pwd = file.read()
-            file.close()
+            with open(pwdfile, 'rb') as file:
+                pwd = file.read()
             self.vncserver_port = self.find_unused_port()
             self.vncserver_pwd = ''.join('\\%o' % c for c in pwd)
             self.vncserver = subprocess.Popen(
@@ -431,7 +456,7 @@ class EpoptesGui(object):
 
     def on_imi_help_home_activate(self, _widget):
         """Handle imi_help_home.activate event."""
-        self.open_url("http://www.epoptes.org")
+        self.open_url("https://epoptes.org")
 
     def on_imi_help_report_bug_activate(self, _widget):
         """Handle imi_help_report_bug.activate event."""
@@ -443,7 +468,7 @@ class EpoptesGui(object):
 
     def on_imi_help_translate_application_activate(self, _widget):
         """Handle imi_help_translate_application.activate event."""
-        self.open_url("http://www.epoptes.org/translations")
+        self.open_url("https://epoptes.org/translations")
 
     def on_imi_help_live_chat_irc_activate(self, _widget):
         """Handle imi_help_live_chat_irc.activate event."""
@@ -829,10 +854,16 @@ class EpoptesGui(object):
                   exc)
             return False
 
+        # Support hiding clients in an X-Hidden group
+        if self.global_groups and 'X-HIDDEN' in self.x_groups:
+            if mac in self.x_groups['X-HIDDEN']:
+                LOG.w("  X-HIDDEN: Won't add this client to my lists")
+                return False
+
         # Check if the incoming client is the same with the computer in which
         # epoptes is running, so we don't add it to the list.
         if (mac in self.current_macs) and ((uid == self.uid) or (uid == 0)):
-            LOG.w("  Won't add this client to my lists")
+            LOG.w("  SAME-PC: Won't add this client to my lists")
             return False
 
         # Compatibility check
@@ -958,10 +989,11 @@ class EpoptesGui(object):
                 except ValueError:
                     LOG.e("Bad thumbshot header")
                     return
-                # TODO: see if there's any way to avoid casting to GLib.Bytes
+                # GLib.Bytes.new and .copy() avoid memory leak and crash (#110)
                 pxb = GdkPixbuf.Pixbuf.new_from_bytes(
                     GLib.Bytes.new(pixels), GdkPixbuf.Colorspace.RGB, False, 8,
                     width, height, rowstride)
+                pxb = pxb.copy()
                 self.current_thumbshots[handle] = pxb
                 self.cstore[i.path][C_PIXBUF] = pxb
                 return
