@@ -1,5 +1,5 @@
 #!/usr/bin/python3
-# This file is part of Epoptes, http://epoptes.org
+# This file is part of Epoptes, https://epoptes.org
 # Copyright 2010-2018 the Epoptes team, see AUTHORS.
 # SPDX-License-Identifier: GPL-3.0-or-later
 """
@@ -45,9 +45,8 @@ class ServerContextFactory(ssl.ContextFactory):
 
 def filter_bash(script):
     """Strip comments from client-functions, to save some bandwidth."""
-    file = open(script)
-    functions = file.readlines()
-    file.close()
+    with open(script) as file:
+        functions = file.readlines()
     result = ''
     for line in functions:
         if line.strip() != '' and line.strip()[0] == '#':
